var express        = require('express'),
    routes         = require('./routes'),
    json           = require('body-parser').json,
    urlencoded     = require('body-parser').urlencoded,
    config         = require('./config'),
    async          = require('async'),
    JSLibsHandler  = require('./lib/JSLibsHandler'),
    MVNLibsHandler = require('./lib/MVNLibsHandler');

var jsHandler = new JSLibsHandler();
var mvnHandler = new MVNLibsHandler();

var app = express();
// middlewares
app.use(json());
app.use(urlencoded({extended: true}));

app.use(function (req, res, next) {
    req.shared = req.shared || {};
    req.shared.jsHandler = jsHandler;
    req.shared.mvnHandler = mvnHandler;
    next();
});
// routes
app.get(config.ROUTE_PREFIX+'/load', routes.load);
app.get(config.ROUTE_PREFIX+'/merge', routes.merge);
app.post(config.ROUTE_PREFIX+'/merge', routes.merge);

// 404 middleware
app.use(function(req, res) {
  // if you end-up here, it means that I do not know the given url
  // so for now => redirect to '/' but you can put a custom 404 if you want
  res.send(404, 'Hum, this is not the page you are looking for');
});

// start up
console.log('Kevoree std libs: preparing caches...');
async.parallel([
    function (cb) {
        jsHandler.once('ready', cb);
        jsHandler.once('error', cb);
    },
    function (cb) {
        mvnHandler.once('ready', cb);
        mvnHandler.once('error', cb);
    }
], function (err) {
    if (err) {
        console.log('Kevoree std libs: failed to load caches!');
        throw err;
    }

    console.log('Kevoree std libs: caches ready');
    app.listen(config.PORT, function() {
        console.log('Kevoree Web Service: started @ localhost:' + config.PORT + config.ROUTE_PREFIX);
    });
});
