var npm             = require('npm'),
    async           = require('async'),
    EventEmitter    = require('events').EventEmitter,
    util            = require('util'),
    config          = require('../config'),
    npmSearch       = require('./npm-search'),
    npmView         = require('./npm-view'),
    npmVers         = require('npm-vers');

/**
 * Created by leiko on 07/05/14.
 */
function JSLibsHandler() {
    this.libraries = [];
    this.isReady = false;

    var updateLibs = function () {
        var start = new Date().getTime();
        var tmpLibs = [];
        npmSearch('/^(kevoree-comp|kevoree-chan|kevoree-group|kevoree-node)-[\\w-]+/', function (err, modules) {
            if (err) return this.emit('error', err);

            var tasks = [];
            for (var name in modules) {
                if (modules.hasOwnProperty(name)) {
                    if (modules[name].version.length !== 0 && (modules[name].keywords.indexOf('kevoree-std-lib') !== -1)) {
                        (function (moduleName) {
                            tasks.push(function (cb) {
                                npmView(moduleName, function (err, view) {
                                    if (err) return cb(err);

                                    var splittedName = view.name.split('-');
                                    view.groupID = '';
                                    view.artifactID = view.name;
                                    view.type = splittedName[1];
                                    view.simpleName = splittedName[2];

                                    delete view.name;
                                    delete view.version;

                                    npmVers(moduleName, function (err, result) {
                                        if (err) return cb(err);

                                        if (result.latestRelease) {
                                            view.latest = result.latestRelease.raw;
                                        } else {
                                            view.latest = result.latestPreRelease.raw;
                                        }

                                        tmpLibs.push(view);
                                        cb();
                                    });
                                }.bind(this));
                            }.bind(this));
                        }.bind(this))(name);
                    }
                }
            }

            async.parallel(tasks, function (err) {
                if (err) return this.emit('error', err);

                console.log('['+new Date().toUTCString()+'] Javascript libraries updated: ' + ((new Date().getTime()) - start) + 'ms');
                this.libraries = tmpLibs;
                if (!this.isReady) {
                    this.emit('ready');
                    this.isReady = true;
                }
            }.bind(this));
        }.bind(this));
    }.bind(this);

    process.nextTick(updateLibs);
    setInterval(updateLibs, config.JS_LIBS_UPDATE);
}

JSLibsHandler.prototype = Object.create(EventEmitter.prototype);

JSLibsHandler.prototype.get = function () {
    return this.libraries;
};


module.exports = JSLibsHandler;