var sonatypeList    = require('./sonatype-list'),
    async           = require('async'),
    EventEmitter    = require('events').EventEmitter,
    config          = require('../config');

/**
 * Created by leiko on 04/06/14.
 */
function MVNLibsHandler() {
    this.libraries = {};
    this.isReady = false;

    var updateLibs = function () {
        var start = new Date().getTime();
        async.parallel([
            function (cb) {
                sonatypeList('java', function (err, libraries) {
                    if (err) return cb(err);
                    this.libraries['java'] = libraries;
                    cb();
                }.bind(this));
            }.bind(this),
            function (cb) {
                sonatypeList('cloud', function (err, libraries) {
                    if (err) return cb(err);
                    this.libraries['cloud'] = libraries;
                    cb();
                }.bind(this));
            }.bind(this)
        ], function (err) {
            if (err) {
                this.emit('error', err);
                return;
            }

            console.log('['+new Date().toUTCString()+'] Java & Cloud libraries updated: ' + ((new Date().getTime()) - start) + 'ms');
            if (!this.isReady) {
                this.emit('ready');
                this.isReady = true;
            }
        }.bind(this));
    }.bind(this);

    process.nextTick(updateLibs);
    setInterval(updateLibs, config.MVN_LIBS_UPDATE);
}

MVNLibsHandler.prototype = Object.create(EventEmitter.prototype);

MVNLibsHandler.prototype.get = function (platform) {
    return this.libraries[platform];
};

module.exports = MVNLibsHandler;