var https = require('https');
var url = require('url');
var path = require('path');
var tarball = require('tarball-extract');
var mkdirp = require('mkdirp');
var config = require('../config');
var fs = require('fs');
var kevoree = require('kevoree-library').org.kevoree;
var npmVers = require('npm-vers');

var jsLibsPath = path.resolve('..', config.JS_LIBS_PATH);
/**
 * Created by leiko on 17/06/14.
 */
function loadModel(modelPath, callback) {
    fs.readFile(modelPath, {encoding: 'utf8'}, function (err, data) {
        if (err) {
            callback(err);

        } else {
            // lib is installed => read its model
            try {
                var factory = new kevoree.factory.DefaultKevoreeFactory();
                var loader = factory.createJSONLoader();
                var model = loader.loadModelFromString(data).get(0);
                callback(null, model);

            } catch (err) {
                var e = new Error('Unable to load JSON model from '+modelPath);
                e.stack = err.stack;
                callback(e);
            }
        }
    });
}

module.exports = function (module, version, callback) {
    version = version || 'latest';

    function doGetModel(module, version) {
        loadModel(path.resolve(jsLibsPath, module+'@'+version, 'package', 'kevlib.json'), function (err, model) {
            if (err) {
                https.get({
                    host: 'registry.npmjs.org',
                    path: '/'+module+'/'+version
                }, function (result) {
                    var data = '';
                    result.on('data', function (chunk) { data += chunk; });
                    result.on('end', function () {
                        var result = JSON.parse(data);
                        if (result.dist) {
                            url = url.parse(result.dist.tarball);
                            var name = path.resolve(jsLibsPath, module+'@'+result.version);
                            // if the first given version was "latest", now pkg.version is the real "latest" version
                            // so let's re-check if we have this module to avoid useless tar extraction
                            loadModel(path.resolve(jsLibsPath, module+'@'+result.version, 'package', 'kevlib.json'), function (err, model) {
                                if (err) {
                                    mkdirp(jsLibsPath, function () {
                                        tarball.extractTarballDownload(url.href, name+'.tgz', name, {}, function (err, result) {
                                            if (err) {
                                                callback(err);
                                                return;
                                            }

                                            fs.unlinkSync(result.downloadFile);
                                            loadModel(path.resolve(result.destination, 'package', 'kevlib.json'), callback);
                                        });
                                    });
                                } else {
                                    callback(null, model);
                                }
                            });
                        } else if (result.error) {
                            // npm registry error
                            callback(new Error('Unable to retrieve model for '+module+'@'+version+' ('+result.error+')'));
                        } else {
                            // unknown error
                            callback(new Error('Unable to retrieve model for '+module+'@'+version+' (unknown error)'));
                        }
                    });

                }).on('error', function (err) {
                    callback(err);
                });

            } else {
                callback(null, model);
            }
        });
    }

    if (version === 'release') {
        npmVers(module, function (err, result) {
            if (err) {
                callback(new Error('Unable to retrieve version for '+module+'@release ('+err.message+')'));
                return;
            }

            if (result.latestRelease) {
                doGetModel(module, result.latestRelease);
            } else {
                callback(new Error('No release version found for '+module));
            }
        });
    } else {
        doGetModel(module, version);
    }
};
