var http        = require('https');
var semver      = require('semver');
var parseString = require('xml2js').parseString;
var querystring = require('querystring');
var async       = require('async');

/**
 * Created by leiko on 04/06/14.
 */
module.exports = function (platform, callback) {
    var latestVersions = {
        host: 'oss.sonatype.org',
        path: '/service/local/lucene/search?' + querystring.stringify({
            g: 'org.kevoree.library.'+platform,
            p: 'jar'
        })
    };
    http.get(latestVersions, function (result) {
        var data = '';
        result.on('data', function (chunk) {
            data += chunk;
        });
        result.on('end', function () {
            // XML data fully retrieved from remote server => parsing and sending result
            parseString(data, function (err, xml) {
                if (err) {
                    console.error('ERROR: /load (platform: '+platform+')');
                    console.error('  > http://'+latestVersions.host+latestVersions.path);
                    console.error(err.stack);
                    callback(new Error('Server unable to parse XML file from Sonatype.'));
                    return;
                }

                var artifacts   = xml['searchNGResponse']['data'][0]['artifact'],
                    libs        = {},
                    tasks       = [];

                for (var i=0; i < artifacts.length; i++) {
                    (function (lib) {
                        if (!libs[lib.artifactId[0]]) {
                            // add a new library for this artifact
                            var splittedArt = lib.artifactId[0].split('.');
                            var latest;
                            if (lib.latestRelease && lib.latestRelease[0]) {
                                latest = lib.latestRelease[0];
                            } else {
                                latest = lib.latestSnapshot[0];
                            }
                            libs[lib.artifactId[0]] = {
                                groupID:    lib.groupId[0],
                                artifactID: lib.artifactId[0],
                                simpleName: splittedArt[splittedArt.length-1],
                                versions:   [],
                                latest:     latest
                            };

                            for (var j=0; j < artifacts.length; j++) {
                                if (artifacts[j].artifactId[0] === lib.artifactId[0]) {
                                    if (libs[lib.artifactId[0]].versions.indexOf(artifacts[j].version[0]) === -1) {
                                        // no duplicate
                                        libs[lib.artifactId[0]].versions.push(artifacts[j].version[0]);
                                    }
                                }
                            }
                        }
                    })(artifacts[i]);
                }

                async.parallel(tasks, function (err) {
                    callback(err, libs);
                });
            });
        });

    }).on('error', function (e) {
        // unable to reach oss.sonatype.org maven repository
        console.log('Unable to reach '+latestVersions.host+': ' + e.message+'. Loading '+platform+' core libraries failed.');
        // display error
        callback(new Error('Unable to reach '+latestVersions.host+': ' + e.message+'. Loading '+platform+' core libraries failed.'));
    });
};