var async       = require('async'),
    getModel    = require('../../lib/getModel'),
    kevoree     = require('kevoree-library').org.kevoree;

var factory    = new kevoree.factory.DefaultKevoreeFactory(),
    compare    = factory.createModelCompare();

module.exports = function (repos, libraries, callback) {
    var asyncTasks  = [];
    var mergedModel = factory.createContainerRoot();

    for (var i=0; i < libraries.length; i++) {
        (function (lib) {
            asyncTasks.push(function (taskCb) {
                getModel(lib.artifactID, lib.version, function (err, model) {
                    if (err) {
                        return taskCb(err);
                    }

                    // model retrieved successfully
                    // now merging it into mergedModel
                    compare.merge(mergedModel, model).applyOn(mergedModel);
                    taskCb();
                });
            });
        })(libraries[i]);
    }

    async.series(asyncTasks, function (err) {
        if (err) return callback(err);

        // all tasks have been executed successfully
        return callback(null, mergedModel);
    });
};