var java = require('java');
var config = require('../../config');
var kevoree = require('kevoree-library').org.kevoree;

java.classpath.push(config.KEV_JAR);

var resolver   = java.newInstanceSync('org.kevoree.resolver.MavenResolver'),
    factory    = new kevoree.factory.DefaultKevoreeFactory(),
    loader     = factory.createJSONLoader(),
    compare    = factory.createModelCompare();

module.exports = function (repos, libraries, callback) {
    var list = java.newInstanceSync('java.util.ArrayList');

    if (repos) {
        for (var i in repos) {
            list.addSync(repos[i]);
        }
    }

    if (list.sizeSync() == 0) {
        // add sonatype as a default central repo if there is 0 repo given (just in case)
        list.addSync("https://oss.sonatype.org/content/groups/public");
    }

    var mergedModel = factory.createContainerRoot();
    for (var i in libraries) {
        var artID   = libraries[i].artifactID,
            grpID   = libraries[i].groupID,
            version = libraries[i].version;

        if (artID && grpID && version) {
            var file = resolver.resolveSync(grpID, artID, version, 'jar', list);
            if (file == null) {
                console.error("Jar file for "+artID+" is null :/");

            } else {
                var jar      = java.newInstanceSync('java.util.jar.JarFile', file),
                    jarEntry = jar.getJarEntrySync("KEV-INF/lib.json"),
                    strWriter = java.newInstanceSync('java.io.StringWriter');

                if (jarEntry != null) {
                    // filling up strWriter with jarEntry input stream content
                    java.callStaticMethodSync('org.apache.commons.io.IOUtils', 'copy', jar.getInputStreamSync(jarEntry), strWriter);
                    // load model from strWriter string
                    var model    = loader.loadModelFromString(strWriter.toStringSync()).get(0),
                        mergeSeq = compare.merge(mergedModel, model);

                    try {
                        mergeSeq.applyOn(mergedModel);
                    } catch (err) {
                        console.error("mergeSeq.applyOn error");
                        console.error(err);
                        return callback(new Error("Something went wrong while merging model."));
                    }

                } else {
                    console.error("JarEntry KEV-INF/lib.kev for "+artID+" doesn't exist :/");
                }
            }

        } else {
            console.error('Malformed request. Library objects must have artifactID, groupID and version parameters');
        }
    }

    return callback(null, mergedModel);
}